"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

require("../../injector");

var _Metadata = _interopRequireDefault(require("../../Metadata"));

var _static = _interopRequireDefault(require("../../Metadata/static"));

var _create = require("../../codec/create");

var _Call = _interopRequireDefault(require("../../primitive/Generic/Call"));

var _SignedBlock = _interopRequireDefault(require("../../json/SignedBlock.002.json"));

var _SignedBlock004Immortal = _interopRequireDefault(require("../../json/SignedBlock.004.immortal.json"));

var _SignedBlock004Mortal = _interopRequireDefault(require("../../json/SignedBlock.004.mortal.json"));

var _SignedBlock2 = _interopRequireDefault(require("../../json/SignedBlock.005.json"));

// Copyright 2017-2019 @polkadot/types authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
describe('SignedBlock', () => {
  beforeEach(() => {
    _Call.default.injectMetadata(new _Metadata.default(_static.default));
  });
  it('decodes a full block', () => {
    const s = (0, _create.createType)('SignedBlock', _SignedBlock.default.result);
    expect(s.block.header.stateRoot.toString()).toEqual('0x24bab7fe0fee187a57bd0029ba151e81a0bae0fea96ad9f85c579c1ba9d88948');
    expect(s.block.header.digest.logs.length).toBe(1);
    expect(s.block.header.digest.logs[0].type).toBe('AuthoritiesChange');
    expect(s.block.header.digest.logs[0].value.toJSON()).toEqual(['5EyrU8k6worwXWfW2LajRvS7XF6eta286tMpHeHUwMPpzUwT', '5Dpq7rU6KAZVdRtcegvpCoeJtZzFPzieRkRz4xajRAiMRxCd', '5CCtQPAYtuDXt6H4WMrn5nFNk4gemkVRD3k1r1mA7YiZLPP5', '5F2ABwFm3fUhWMZT7Zeu7UjCr1dndjJD7qBeHChJhiJC99UH', '5CA1n5DSpE46Q9Y7NaqD39T9X7S2dRtscsPvrv2dWuYhFg5U', '5CrxmsTCLbPRzHnFPs8Q6cx8jzHweegMSQYEJBGXLjDZWDNv', '5EQ4ZxYew2CXPkjGCBuPdTVg7Hp3bmBXiXBwn1iDmvj6CY3u']);
  }); // Test to replicate https://github.com/polkadot-js/api/issues/1212

  it('decodes to known extrinsics', () => {
    const s = (0, _create.createType)('SignedBlock', _SignedBlock2.default.result);
    const indexes = s.block.extrinsics.map((_ref) => {
      let {
        method: {
          callIndex
        }
      } = _ref;
      return callIndex;
    });
    expect(indexes).toEqual([new Uint8Array([0x02, 0x00]), new Uint8Array([0x0c, 0x00])]);
  });
  describe('eras', () => {
    it('can decode immortals', () => {
      const s = (0, _create.createType)('SignedBlock', _SignedBlock004Immortal.default.result);
      const immortalTx = s.block.extrinsics[0];
      expect(immortalTx.method.methodName).toEqual('transfer');
      expect(immortalTx.method.sectionName).toEqual('balances');
    });
    it('can decode mortals', () => {
      const s = (0, _create.createType)('SignedBlock', _SignedBlock004Mortal.default.result);
      const mortalTx = s.block.extrinsics[0];
      expect(mortalTx.method.methodName).toEqual('transfer');
      expect(mortalTx.method.sectionName).toEqual('balances');
    });
  });
});